#include <avr/io.h>
#include <avr/interrupt.h>
//Counts overflovs
volatile uint16_t T1Ovs1, T1Ovs2;
//Variables holding three timestamps
volatile uint16_t Capt1, Capt2, Capt3; 
//capture Flag
volatile uint8_t Flag;
//Initialize timer
void InitTimer1(void)
{
//Set Initial Timer value
TCNT1=0;
//First capture on rising edge
TCCR1B|=(1<<ICES1);
//Enable input capture and overflow interrupts
TIMSK1|=(1<<ICIE1)|(1<<TOIE1);
}
void StartTimer1(void)
{
//Start timer without prescaller
TCCR1B|=(1<<CS10);
//Enable global interrutps
sei();
}

//capture ISR
ISR(TIMER1_CAPT_vect)
{
if (Flag==0)
	{
		//save captured timestamp
		Capt1=ICR1;
		//change capture on falling edge
		TCCR1B&=~(1<<ICES1);
		//reset overflows
		T1Ovs2=0;
	}
if (Flag==1)
	{
		Capt2=ICR1;
		//change capture on rising edge
		TCCR1B|=(1<<ICES1);
		//save first overflow counter
		T1Ovs1=T1Ovs2;
	}
if (Flag==2)
	{
		Capt3=ICR1;
		//stop input capture and overflow interrupts
		TIMSK1&=~((1<<ICIE1)|(1<<TOIE1));
	}
//increment Flag
Flag++;
}
//Overflow ISR
ISR(TIMER1_OVF_vect)
{
//increment overflow counter
T1Ovs2++;
}
int main(void)
{
//dutycycle result holder
volatile uint8_t DutyCycle;
InitTimer1();
StartTimer1();
	while(1)
	{
		//calculate duty cycle if all timestamps captured
		if (Flag==3)
		{
				DutyCycle=(uint8_t)((((uint32_t)(Capt2-Capt1)+((uint32_t)T1Ovs1*0x10000L))*100L) 
					/((uint32_t)(Capt3-Capt1)+((uint32_t)T1Ovs2*0x10000L)));
				//send Duty Cycle value to LCD or USART
				//clear flag
				Flag=0;	
				//clear overflow counters;
				T1Ovs1=0;
				T1Ovs2=0;
				//clear interrupt flags to avoid any pending interrupts
				TIFR1|=(1<<ICF1)|(1<<TOV1);
				//enable input capture and overflow interrupts 
				TIMSK1|=(1<<ICIE1)|(1<<TOIE1);
		}
	}
}
